@@ -33,6 +33,8 @@ class Op:
         return self.__class__ == op.__class__ and self.args == op.args
     def scrub(self, node):
         return node
+    def is_slice(self):
+        return False
 
 
 class MetaNOP(type):
@@ -47,6 +49,8 @@ class NOP(metaclass=MetaNOP):
     @classmethod
     def matches(cls, vals):
         return ()
+    def is_slice(self):
+        return False
 
 
 class Const(Op):
@@ -355,11 +359,29 @@ class Slice(Op):
         return '[' + ':'.join(m) + ']'
     def is_pattern(self):
         return False
+    def is_slice(self):
+        return True
     def operator(self, top=False):
         return str(self)
-    def slice(self, node):
-        args = ( len(node) if a == '+' else a for a in self.args )
+    def slice(self, node=None):
+        args = self.args
+        if node is not None:
+            args = ( len(node) if a == '+' else a for a in self.args )
         return slice(*args)
+    def cardinality(self, node=None):
+        """
+        Calculate cardinality of a slice; don't both dealing with countably infinite
+        set arithmetic, instead just pick a suitably large integer
+        """
+        s = self.slice(node)
+        start = s.start or 0
+        stop = s.stop or '+'
+        step = s.step or 1
+        if '+' in (start, step):
+            return 0
+        if stop == '+':
+            return 1 << 64
+        return max(0, int((stop - start) / step))
     def keys(self, node):
         return (self.slice(node),)
     def items(self, node):
@@ -370,12 +392,12 @@ class Slice(Op):
         return not node[self.slice(node)]
     def default(self):
         return []
-
     def match(self, op, specials=False):
         if not isinstance(op, Slice):
             return None
-        return Match(op.slice) if self.slice == op.slice else None
-
+        if self.cardinality() < op.cardinality():
+            return None
+        return Match(op.slice())
     def update(self, node, key, val):
         if node[key] == val:
             return node
